/****************************************************************************
 * This file is part of the project AqHome.
 * AqHome (c) by 2025 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "./getvalues.h"
#include "../client.h"
#include "../utils.h"

#include "aqhome/aqhome.h"
#include "aqhome/msg/ipc/m_ipc.h"
#include "aqhome/msg/ipc/m_ipc_result.h"
#include "aqhome/msg/ipc/data/m_ipcd.h"
#include "aqhome/msg/ipc/data/m_ipcd_values.h"
#include "aqhome/msg/ipc/data/m_ipcd_getvalues.h"

#include <gwenhywfar/args.h>
#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/text.h>



/* ------------------------------------------------------------------------------------------------
 * defs
 * ------------------------------------------------------------------------------------------------
 */

#define I18S(msg) msg
#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)

#define A_ARG GWEN_ARGS_FLAGS_HAS_ARGUMENT
#define A_END (GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST)
#define A_CHAR GWEN_ArgsType_Char
#define A_INT GWEN_ArgsType_Int


/* ------------------------------------------------------------------------------------------------
 * forward declarations
 * ------------------------------------------------------------------------------------------------
 */

static AQH_MESSAGE *_createRequestMessage(AQH_OBJECT *o, uint32_t msgId);
static int _handleResponseMessage(AQH_OBJECT *o, const AQH_MESSAGE *msg, const GWEN_TAG16_LIST *tagList, int first);



/* ------------------------------------------------------------------------------------------------
 * code
 * ------------------------------------------------------------------------------------------------
 */

int AQH_Tool_GetValues(GWEN_DB_NODE *dbGlobalArgs, int argc, char **argv)
{
  AQH_EVENT_LOOP *eventLoop;
  AQH_OBJECT *o;
  int rv;
  const GWEN_ARGS args[]= {
    /* flags type    name              min max s    long           short_descr, long_descr */
    { A_ARG, A_CHAR, "brokerAddress",  0,  1,  "t", "tcpaddress",  I18S("TCP address to connect to [127.0.0.1]"), NULL},
    { A_ARG, A_INT,  "brokerPort",     0,  1,  "P", "tcpport",     I18S("Specify the TCP port to listen on"), NULL},
    { A_ARG, A_INT,  "timeout",        0,  1,  "T", NULL,          I18S("Specify timeout in seconds for response"), NULL},
    { A_ARG, A_CHAR, "brokerClientId", 0,  1,  "c", "clientid",    I18S("Specify CLIENTID"), NULL},
    { A_ARG, A_CHAR, "userId",         0,  1,  "u", "userid",      I18S("Specify user id"), NULL},
    { A_ARG, A_CHAR, "device",         0,  1,  "d", "device",      I18S("device name to match"), NULL},
    { A_ARG, A_CHAR, "modality",       0,  1,  "m", NULL,          I18S("Modality to match"), NULL},
    { A_ARG, A_CHAR, "password",       0,  1,  "p", "password",    I18S("Specify service password"), NULL},
    { 0,     A_INT,  "printHeader",    0,  1,  "H", "printheader", I18S("Print header if given"), NULL},
    { A_END, A_INT,  "help",           0,  0,  "h", "help",        I18S("Show this help screen"), NULL}
  };

  eventLoop=AQH_EventLoop_new();
  o=AQH_ToolClient_new(eventLoop, AQH_IPC_PROTOCOL_DATA_ID, AQH_IPC_PROTOCOL_DATA_VERSION, dbGlobalArgs, args);
  AQH_ToolClient_AddFlags(o, AQH_TOOL_CLIENT_CONNECTFLAGS_WITHCONNECTMSG);
  AQH_ToolClient_SetCreateRequestMessageFn(o, _createRequestMessage);
  AQH_ToolClient_SetHandleResponseMessageFn(o, _handleResponseMessage);
  rv=AQH_ToolClient_ReadLocalArgs(o, argc, argv);
  if (rv!=0)
    return rv;
  rv=AQH_ToolClient_Run(o);
  AQH_Object_free(o);
  AQH_EventLoop_free(eventLoop);
  return rv;
}



AQH_MESSAGE *_createRequestMessage(GWEN_UNUSED AQH_OBJECT *o, uint32_t msgId)
{
  GWEN_DB_NODE *dbArgs;
  const char *deviceName;
  const char *s;
  int modality;

  dbArgs=AQH_ToolClient_GetDbLocalArgs(o);
  deviceName=GWEN_DB_GetCharValue(dbArgs, "device", 0, NULL);
  s=GWEN_DB_GetCharValue(dbArgs, "modality", 0, NULL);
  if (s && *s) {
    modality=AQH_ValueModality_fromString(s);
    if (modality==AQH_ValueModality_Unknown) {
    }
  }
  else
    modality=AQH_ValueModality_Unknown;

  return AQH_IpcdMessageGetValues_new(AQH_MSGTYPE_IPC_DATA_GETVALUES_REQ, msgId, 0, deviceName, modality);
}



int _handleResponseMessage(AQH_OBJECT *o, const AQH_MESSAGE *msg, const GWEN_TAG16_LIST *tagList, int first)
{
  uint16_t code;
  GWEN_DB_NODE *dbArgs;
  int printHeader;

  dbArgs=AQH_ToolClient_GetDbLocalArgs(o);
  printHeader=first?GWEN_DB_GetIntValue(dbArgs, "printHeader", 0, 0):0;

  code=AQH_IpcMessage_GetCode(msg);
  if (code==AQH_MSGTYPE_IPC_DATA_GETVALUES_RSP) {
    AQH_VALUE_LIST *valueList;

    valueList=AQH_IpcdMessageValues_ReadValueList(tagList);
    if (valueList) {
      AQH_VALUE *value;

      value=AQH_Value_List_First(valueList);
      while(value) {
        Utils_PrintValue(value, printHeader);
        printHeader=0;
        value=AQH_Value_List_Next(value);
      }
      AQH_Value_List_free(valueList);
    }

    if (AQH_IpcdMessageValues_GetFlags(tagList) & AQH_MSGDATA_VALUES_FLAGS_LASTMSG) {
      DBG_INFO(NULL, "Last message received");
      return 1;
    }
    return 0;
  }
  else if (code==AQH_MSGTYPE_IPC_DATA_RESULT)
    return AQH_ToolClient_HandleResultMsg(msg, tagList, first);
  else {
    DBG_INFO(NULL, "Unexpected message \"%d\"", code);
    return 3;
  }
}


